# -*- coding: utf8 -*-
from __future__ import unicode_literals

import random

from driverng.protocol.astm import AstmProtocol
from driverng.transport.base import TransportException
from driverng.utils import ACK, ENQ, EOT, byte2log


class AstmSpecificProtocol(AstmProtocol):
    def send_trames(self, trames):
        """Envoie une liste de trame (message) sur la connexion.
        Les trames sont construites avec `build_trames` puis envoyés
        suivant le protocole automate::

            -> <enq>
            <- <ack>
            -> DATA
            <- <ack>
            -> <eot>

        :return: False si erreur sinon True (a traiter : exceptions)
        :param trames: liste de trames à envoyer
        """

        self.write(ENQ)
        try:
            data = self.read(timeout=15)
        except TransportException:
            self.logger.info("Aucun acquittement dans le timeout de 15 secondes")
            self.logger.info(
                "L'envoi de la demande (ou réponse au query) n'a pas pu etre effectué"
            )
            # Il faut passer à autre chose, l'automate (KIESTRA ou WASPLAB) ne répondra plus
            return True
        if not data:
            self.write(EOT)
            return False

        if data[0] == ENQ:
            # L'automate a la priorité, le driver attends un temps
            # aléatoire entre 10 et 20 secondes
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "ENQ clash: wait %ds before re-sending <enq>", self.rtimeout
            )
            return False

        if data[0] != ACK:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<ack> expected in %s: wait %ds before re-sending <enq>",
                byte2log(data),
                self.rtimeout,
            )
            return False

        index = 0

        while True:
            try:
                trame = trames[index]
            except IndexError:
                break

            self.write(trame)
            data = self.read(timeout=15)

            if not data:
                self.write(EOT)
                return False

            if data[0] not in (ACK, EOT):
                self.rtimeout = random.randint(10, 20)
                self.logger.info(
                    "<ack> or <eot> expected in %s: wait %ds before re-sending <enq>",
                    byte2log(data),
                    self.rtimeout,
                )
                self.write(EOT)
                return False

            index += 1

        self.write(EOT)

        return True
